@@ -27,27 +27,33 @@
 using System;
 using System.IO;
 using System.Text;
+
 using Beagle.Util;
+using Beagle.Daemon;
+
+using SemWeb;
 
 namespace Beagle.Filters {
 	
-	public class FilterJpeg : Beagle.Daemon.Filter {
+	public class FilterJpeg : FilterImage {
 
-		public FilterJpeg ()
+		public FilterJpeg () : base ()
 		{
-			AddSupportedMimeType ("image/jpeg");
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("image/jpeg"));
 		}
 
 		// FIXME: This is not particularly efficient
-		protected override void DoPullProperties ()
+		protected override void PullImageProperties ()
 		{
 			JpegHeader header = new JpegHeader (Stream);
 			byte [] commentdata = header.GetJFIFComment();
 			if (commentdata != null && commentdata.Length != 0)
 			{
-				string comment = System.Text.Encoding.ASCII.GetString( commentdata, 0, commentdata.Length );
-				if (comment != null && comment != "")
+				string comment = System.Text.Encoding.Default.GetString( commentdata, 0, commentdata.Length );
+				if (comment != null && comment != "") {
 					AddProperty (Beagle.Property.New ("jfif:Comment", comment));
+					AddProperty (Beagle.Property.NewUnstored ("fixme:comment", comment));
+				}
 			}
 
 			byte [] data = header.GetRawExif ();
@@ -60,20 +66,29 @@ namespace Beagle.Filters {
 			string str;
 			
 			str = exif.LookupFirstValue (ExifTag.UserComment);
-			if (str != null && str != "")
+			if (str != null && str != "") {
 				AddProperty (Beagle.Property.New ("exif:UserComment", str));
+				AddProperty (Beagle.Property.NewUnstored ("fixme:comment", str));
+			}
+
 
 			str = exif.LookupFirstValue (ExifTag.ImageDescription);
-			if (str != null && str != "")
+			if (str != null && str != "") {
 				AddProperty (Beagle.Property.New ("exif:ImageDescription", str));
+				AddProperty (Beagle.Property.NewUnstored ("fixme:comment", str));
+			}
 
 			str = exif.LookupFirstValue (ExifTag.PixelXDimension);
-			if (str != null && str != "")
+			if (str != null && str != "") {
+				Width = Int32.Parse (str);
 				AddProperty (Beagle.Property.NewKeyword ("exif:PixelXDimension", str));
+			}
 
 			str = exif.LookupFirstValue (ExifTag.PixelYDimension);
-			if (str != null && str != "")
+			if (str != null && str != "") {
+				Height = Int32.Parse (str);
 				AddProperty (Beagle.Property.NewKeyword ("exif:PixelYDimension", str));
+			}
 
 			str = exif.LookupFirstValue (ExifTag.ISOSpeedRatings);
 			if (str != null && str != "")
@@ -121,6 +136,20 @@ namespace Beagle.Filters {
 				}
 			}
 
+			byte [] xmp_data = header.GetRawXmp ();
+
+			if (xmp_data != null) {
+				int i;
+				for (i = 0; i < xmp_data.Length; i++)
+					if (xmp_data [i] == 0) {
+						i++;
+						break;
+					}
+
+				XmpFile xmp = new XmpFile (new MemoryStream (xmp_data, i, xmp_data.Length - i));
+				AddXmpProperties (xmp);
+			}
+
 			Finished (); // That's all folks...
 		}
 	}
