@@ -87,6 +87,7 @@ namespace Beagle.Filters {
 				}
 			}
 			pout.Close ();
+			pc.WaitForExit ();
 			pc.Close ();
 		}
 		
@@ -96,7 +97,7 @@ namespace Beagle.Filters {
 			Process pc = new Process ();
 			pc.StartInfo.FileName = "pdftotext";
 			// FIXME: We probably need to quote special chars in the path
-			pc.StartInfo.Arguments = String.Format ("-nopgbrk -enc UTF-8 \"{0}\" -", FileInfo.FullName);
+			pc.StartInfo.Arguments = String.Format ("-q -nopgbrk -enc UTF-8 \"{0}\" -", FileInfo.FullName);
 			pc.StartInfo.RedirectStandardInput = false;
 			pc.StartInfo.RedirectStandardOutput = true;
 			pc.StartInfo.UseShellExecute = false;
@@ -108,6 +109,9 @@ namespace Beagle.Filters {
 				return;
 			}
 
+			// Nice the process so we don't monopolize the CPU as much; we might want to even set this to idle
+			pc.PriorityClass = ProcessPriorityClass.BelowNormal;	
+
 			// add pdftotext's output to pool
 			StreamReader pout = pc.StandardOutput;
 
@@ -120,8 +124,10 @@ namespace Beagle.Filters {
 				AppendStructuralBreak ();
 			}
 			pout.Close ();
+			pc.WaitForExit ();
 			pc.Close ();
 			Finished ();
 		}
 	}
 }
+
