@@ -105,12 +105,17 @@ namespace Beagle.Filters {
 				Logger.Log.Debug ("Exception occurred while indexing {0}.", FileInfo.FullName);
 				Logger.Log.Debug (e);
 			}
+			pout.Close ();
+			pc.WaitForExit ();
+			pc.Close ();
 			Finished ();
 		}
 		
 		override protected void DoClose ()
 		{
-			xmlReader.Close ();
+			if (xmlReader != null)
+				xmlReader.Close ();
 		}
 	}
 }
+
