@@ -555,5 +555,62 @@ namespace Beagle.Util {
 
 			return email;
 		}
+
+		/**
+		 * expands environment variables in a string e.g.
+		 * folders=$HOME/.kde/share/...
+		 */
+		public static string ExpandEnvVariables (string path)
+		{
+			int dollar_pos = path.IndexOf ('$');
+			if (dollar_pos == -1)
+				return path;
+			
+			System.Text.StringBuilder sb = 
+				new System.Text.StringBuilder ( (dollar_pos == 0 ? "" : path.Substring (0, dollar_pos)));
+			
+			while (dollar_pos != -1 && dollar_pos + 1 < path.Length) {
+				// FIXME: kconfigbase.cpp contains an additional case, $(expression)/.kde/...
+				// Ignoring such complicated expressions for now. Volunteers ;) ?
+				int end_pos = dollar_pos;
+				if (path [dollar_pos + 1] != '$') {
+					string var_name;
+					end_pos ++;
+					if (path [end_pos] == '{') {
+						while ((end_pos < path.Length) && 
+						       (path [end_pos] != '}'))
+							end_pos ++;
+						end_pos ++;
+						var_name = path.Substring (dollar_pos + 2, end_pos - dollar_pos - 3);
+					} else {
+						while ((end_pos < path.Length) &&
+						       (Char.IsNumber (path [end_pos]) ||
+							Char.IsLetter (path [end_pos]) ||
+							path [end_pos] == '_'))
+							end_pos ++;
+						var_name = path.Substring (dollar_pos + 1, end_pos - dollar_pos - 1);
+					}
+					string value_env = null;
+					if (var_name != String.Empty)
+						value_env = Environment.GetEnvironmentVariable (var_name);
+					if (value_env != null) {
+						sb.Append (value_env);
+					}
+					// else, no environment variable with that name exists. ignore
+				}else // else, ignore the first '$', second one will be expanded
+					end_pos ++;
+				if (end_pos >= path.Length)
+					break;
+				dollar_pos = path.IndexOf ('$', end_pos);
+				if (dollar_pos == -1) {
+					sb.Append (path.Substring (end_pos));
+				} else {
+					sb.Append (path.Substring (end_pos, dollar_pos - end_pos));
+				}
+			}
+
+			return sb.ToString ();
+		}
+
 	}
 }
