@@ -41,6 +41,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 	public abstract class EvolutionMailIndexableGenerator : IIndexableGenerator {
 
+		protected static bool Debug = false;
+
 		private static bool gmime_initialized = false;
 
 		private static ArrayList excludes = new ArrayList ();
@@ -269,13 +271,21 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 					return null;
 				}
 				
+				// This extracts the UID and flags from the X-Evolution header.
+				// It may also contain user-defined flags and tags, but we don't
+				// support those right now.
 				int separator_idx = x_evolution.IndexOf ('-');
 
 				string uid_str = x_evolution.Substring (0, separator_idx);
 				string uid = Convert.ToUInt32 (uid_str, 16).ToString (); // ugh.
-				uint flags = Convert.ToUInt32 (x_evolution.Substring (separator_idx + 1), 16);
+				uint flags = Convert.ToUInt32 (x_evolution.Substring (separator_idx + 1, 4), 16);
 
 				Indexable indexable = this.GMimeMessageToIndexable (uid, message, flags);
+
+				if (Debug) {
+					Logger.Log.Debug ("Constructed message {0} with uid {1}, flags {2}.  Indexable {3} null",
+							  this.count, uid, flags, indexable == null ? "" : "not");
+				}
 				
 				if (indexable == null)
 					return null;
@@ -335,15 +345,9 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			if (this.folder_name == "Sent")
 				indexable.AddProperty (Property.NewFlag ("fixme:isSent"));
 
-			//indexable.AddProperty (Property.NewDate ("fixme:date", message.Date));
-#if false
-			if (this.folder_name == "Sent")
-				indexable.AddProperty (Property.NewDate ("fixme:sentdate", message.Date));
-			else
-				indexable.AddProperty (Property.NewDate ("fixme:received", message.Date));
-#endif
-
-			indexable.AddProperty (Property.NewKeyword ("fixme:flags", flags));
+	                Property flag_prop = Property.NewKeyword ("fixme:flags", flags);
+			flag_prop.IsMutable = true;
+			indexable.AddProperty (flag_prop);
 
 			if (CheckFlags (flags, Camel.CamelFlags.Answered))
 				indexable.AddProperty (Property.NewFlag ("fixme:isAnswered"));
@@ -363,10 +367,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			if (CheckFlags (flags, Camel.CamelFlags.AnsweredAll))
 				indexable.AddProperty (Property.NewFlag ("fixme:isAnsweredAll"));
 
-			MemoryStream stream = new MemoryStream ();
-			message.WriteToStream (stream);
-			stream.Seek (0, SeekOrigin.Begin);
-			indexable.SetBinaryStream (stream);
+			indexable.SetBinaryStream (message.Stream);
 
 			return indexable;
 		}
@@ -686,10 +687,16 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 			++this.count;
 
+			if (Debug) {
+				Logger.Log.Debug ("Constructed message {0} with uid {1}, flags {2}.",
+						  this.count, mi.uid, mi.flags);
+			}
+
 			// Try to load the cached message data off disk
 			object flags = this.mapping[mi.uid];
 
-			if (flags == null || (uint) flags != mi.flags) {
+			if (flags == null) {
+				// New, previously unseen message
 				string msg_file;
 
 				if (this.backend_type == ImapBackendType.Imap)
@@ -704,13 +711,33 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 				indexable = this.CamelMessageToIndexable (mi, msg_file);
 
+				if (Debug)
+					Logger.Log.Debug ("Unseen message, indexable {0} null", indexable == null ? "" : "not");
+
 				this.mapping[mi.uid] = mi.flags;
 
-				if (indexable != null) {
+				if (indexable != null)
 					++this.indexed_count;
-					this.deleted_list.Remove (mi.uid);
-				}
-			} else if (flags != null)
+			} else if ((uint) flags != mi.flags) {
+				// Previously seen message, but flags have changed.
+				Uri uri = CamelMessageUri (mi);
+				indexable = new Indexable (uri);
+				indexable.Type = IndexableType.PropertyChange;
+
+				Property flag_prop = Property.NewKeyword ("fixme:flags", mi.flags);
+				flag_prop.IsMutable = true;
+				indexable.AddProperty (flag_prop);
+
+				if (Debug)
+					Logger.Log.Debug ("Previously seen message, flags changed: {0} -> {1}", flags, mi.flags);
+
+				++this.indexed_count;
+			} else {
+				if (Debug)
+					Logger.Log.Debug ("Previously seen message, unchanged.");
+			}
+
+			if (flags != null)
 				this.deleted_list.Remove (mi.uid);
 
 			return indexable;
@@ -735,7 +762,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			Uri uri = CamelMessageUri (messageInfo);
 			Indexable indexable = new Indexable (uri);
 
-			indexable.Timestamp = messageInfo.Date;
+			indexable.Timestamp = messageInfo.SentDate;
 			indexable.MimeType = "message/rfc822";
 			indexable.HitType = "MailMessage";
 
@@ -744,8 +771,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			indexable.AddProperty (Property.NewKeyword ("fixme:client", "evolution"));
 			
 			if (!have_content) {
-				indexable.AddProperty (Property.New ("dc:title", messageInfo.subject));
-				indexable.AddProperty (Property.NewDate ("fixme:date", messageInfo.Date));
+				indexable.AddProperty (Property.New ("dc:title", GMime.Utils.HeaderDecodePhrase (messageInfo.subject)));
+				indexable.AddProperty (Property.NewDate ("fixme:date", messageInfo.SentDate));
 			}
 
 			GMime.InternetAddressList addrs;
@@ -753,7 +780,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (!have_content) {
 					indexable.AddProperty (Property.NewKeyword ("fixme:to", ia.ToString (false)));
-					indexable.AddProperty (Property.NewKeyword ("fixme:to_address", ia.Addr));
+					indexable.AddProperty (Property.New ("fixme:to_address", ia.Addr));
+					indexable.AddProperty (Property.New ("fixme:to_sanitized", StringFu.SanitizeEmail (ia.Addr)));
 					indexable.AddProperty (Property.New ("fixme:to_name", ia.Name));
 				}
 
@@ -766,7 +794,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (!have_content) {
 					indexable.AddProperty (Property.NewKeyword ("fixme:cc", ia.ToString (false)));
-					indexable.AddProperty (Property.NewKeyword ("fixme:cc_address", ia.Addr));
+					indexable.AddProperty (Property.New ("fixme:cc_address", ia.Addr));
+					indexable.AddProperty (Property.New ("fixme:cc_sanitized", StringFu.SanitizeEmail (ia.Addr)));
 					indexable.AddProperty (Property.New ("fixme:cc_name", ia.Name));
 				}
 
@@ -779,7 +808,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (!have_content) {
 					indexable.AddProperty (Property.NewKeyword ("fixme:from", ia.ToString (false)));
-					indexable.AddProperty (Property.NewKeyword ("fixme:from_address", ia.Addr));
+					indexable.AddProperty (Property.New ("fixme:from_address", ia.Addr));
+					indexable.AddProperty (Property.New ("fixme:from_sanitized", StringFu.SanitizeEmail (ia.Addr)));
 					indexable.AddProperty (Property.New ("fixme:from_name", ia.Name));
 				}
 
@@ -788,8 +818,11 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			}
 			addrs.Dispose ();
 
-                        indexable.AddProperty (Property.NewKeyword ("fixme:mlist",    messageInfo.mlist));
-                        indexable.AddProperty (Property.NewKeyword ("fixme:flags",    messageInfo.flags));
+                        indexable.AddProperty (Property.NewKeyword ("fixme:mlist", messageInfo.mlist));
+
+			Property flag_prop = Property.NewKeyword ("fixme:flags", messageInfo.flags);
+			flag_prop.IsMutable = true;
+			indexable.AddProperty (flag_prop);
 
 			if (this.folder_name == "Sent")
 				indexable.AddProperty (Property.NewFlag ("fixme:isSent"));
